@@ -115,14 +115,19 @@ class Collection(object):
         """
         return self.__collection_name
 
-    def _send_message(self, operation, data):
+    def _send_message(self, operation, data, safe=False):
         """Wrap up a message and send it.
+
+        If safe is true then piggyback a lasterror message on the send,
+        raising an OperationFailure exception when the insert/update
+        failed.
         """
         # reserved int, full collection name, message data
         message = _ZERO
         message += bson._make_c_string(self.full_name())
         message += data
-        return self.__database.connection()._send_message(operation, message)
+
+        return self.__database.connection()._send_message(operation, message, safe)
 
     def database(self):
         """Get the database that this collection is a part of.
@@ -134,13 +139,14 @@ class Collection(object):
 
         If `to_save` already has an '_id' then an update (upsert) operation
         is performed and any existing document with that _id is overwritten.
-        Otherwise an insert operation is performed. Returns the _id of the
-        saved document.
+        Otherwise an '_id' will be added to `to_save` and an insert operation
+        is performed. Returns the _id of the saved document.
 
         Raises TypeError if to_save is not an instance of dict. If `safe`
         is True then the save will be checked for errors, raising
-        OperationFailure if one occurred. Checking for safety requires an extra
-        round-trip to the database. Returns the _id of the saved document.
+        OperationFailure if one occurred. Safe inserts wait for a
+        response from the database, while normal inserts do not. Returns the
+        _id of the saved document.
 
         :Parameters:
           - `to_save`: the SON object to be saved
@@ -164,9 +170,10 @@ class Collection(object):
         If manipulate is set the document(s) are manipulated using any
         SONManipulators that have been added to this database. Returns the _id
         of the inserted document or a list of _ids of the inserted documents.
+        If the document(s) does not already contain an '_id' one will be added.
         If `safe` is True then the insert will be checked for errors, raising
-        OperationFailure if one occurred. Checking for safety requires an extra
-        round-trip to the database.
+        OperationFailure if one occurred. Safe inserts wait for a response from
+        the database, while normal inserts do not.
 
         :Parameters:
           - `doc_or_docs`: a SON object or list of SON objects to be inserted
@@ -178,19 +185,12 @@ class Collection(object):
         docs = doc_or_docs
         if isinstance(docs, types.DictType):
             docs = [docs]
-        elif not isinstance(docs, types.ListType):
-            raise TypeError("insert takes a document or list of documents")
 
         if manipulate:
             docs = [self.__database._fix_incoming(doc, self) for doc in docs]
 
         data = [bson.BSON.from_dict(doc, check_keys) for doc in docs]
-        self._send_message(2002, "".join(data))
-
-        if safe:
-            error = self.__database.error()
-            if error:
-                raise OperationFailure("insert failed: " + error["err"])
+        self._send_message(2002, "".join(data), safe)
 
         ids = [doc.get("_id", None) for doc in docs]
         return len(ids) == 1 and ids[0] or ids
@@ -202,8 +202,8 @@ class Collection(object):
         Raises TypeError if either spec or document isn't an instance of
         dict or upsert isn't an instance of bool. If `safe` is True then
         the update will be checked for errors, raising OperationFailure if one
-        occurred. Checking for safety requires an extra round-trip to the
-        database.
+        occurred. Safe inserts wait for a response from the database, while
+        normal inserts do not.
 
         :Parameters:
           - `spec`: a SON object specifying elements which must be present for
@@ -231,23 +231,22 @@ class Collection(object):
         message += bson.BSON.from_dict(spec)
         message += bson.BSON.from_dict(document)
 
-        self._send_message(2001, message)
+        self._send_message(2001, message, safe)
 
-        if safe:
-            error = self.__database.error()
-            if error:
-                raise OperationFailure("update failed: " + error["err"])
-
-    def remove(self, spec_or_object_id):
+    def remove(self, spec_or_object_id, safe=False):
         """Remove an object(s) from this collection.
 
         Raises TypeEror if the argument is not an instance of
-        (dict, ObjectId).
+        (dict, ObjectId). If `safe` is True then the remove will be checked for
+        errors, raising OperationFailure if one occurred. Safe removes wait for
+        a response from the database, while normal removes do not.
+
 
         :Parameters:
           - `spec_or_object_id` (optional): a SON object specifying elements
             which must be present for a document to be removed OR an instance
             of ObjectId to be used as the value for an _id element
+          - `safe` (optional): check that the remove succeeded?
         """
         spec = spec_or_object_id
         if isinstance(spec, ObjectId):
@@ -257,7 +256,7 @@ class Collection(object):
             raise TypeError("spec must be an instance of dict, not %s" %
                             type(spec))
 
-        self._send_message(2006, _ZERO + bson.BSON.from_dict(spec))
+        self._send_message(2006, _ZERO + bson.BSON.from_dict(spec), safe)
 
     def find_one(self, spec_or_object_id=None, fields=None, slave_okay=None,
                  _sock=None, _must_use_master=False):
@@ -272,11 +271,9 @@ class Collection(object):
             of ObjectId to be used as the value for an _id query
           - `fields` (optional): a list of field names that should be included
             in the returned document ("_id" will always be included)
-          - `slave_okay` (optional): if True, this query should be allowed to
-            execute on a slave (by default, certain queries are not allowed to
-            execute on mongod instances running in slave mode). If slave_okay
-            is set to None the Connection level default will be used - see the
-            slave_okay parameter to `pymongo.Connection.__init__`.
+          - `slave_okay` (optional): DEPRECATED this option is deprecated and
+            will be removed - see the slave_okay parameter to
+            `pymongo.Connection.__init__`.
         """
         spec = spec_or_object_id
         if spec is None:
@@ -308,7 +305,7 @@ class Collection(object):
         return as_dict
 
     def find(self, spec=None, fields=None, skip=0, limit=0,
-             slave_okay=None, timeout=True, snapshot=False,
+             slave_okay=None, timeout=True, snapshot=False, tailable=False,
              _sock=None, _must_use_master=False):
         """Query the database.
 
@@ -334,11 +331,9 @@ class Collection(object):
           - `skip` (optional): the number of documents to omit (from the start
             of the result set) when returning the results
           - `limit` (optional): the maximum number of results to return
-          - `slave_okay` (optional): if True, this query should be allowed to
-            execute on a slave (by default, certain queries are not allowed to
-            execute on mongod instances running in slave mode). If slave_okay
-            is set to None the Connection level default will be used - see the
-            slave_okay parameter to `pymongo.Connection.__init__`.
+          - `slave_okay` (optional): DEPRECATED this option is deprecated and
+            will be removed - see the slave_okay parameter to
+            `pymongo.Connection.__init__`.
           - `timeout` (optional): if True, any returned cursor will be subject
             to the normal timeout behavior of the mongod process. Otherwise,
             the returned cursor will never timeout at the server. Care should
@@ -347,14 +342,26 @@ class Collection(object):
           - `snapshot` (optional): if True, snapshot mode will be used for this
             query. Snapshot mode assures no duplicates are returned, or objects
             missed, which were present at both the start and end of the query's
-            execution. For details, see the wiki_
-
-        .. _wiki: http://www.mongodb.org/display/DOCS/How+to+do+Snapshotting+in+the+Mongo+Database
+            execution. For details, see the `wiki
+            <http://www.mongodb.org/display/DOCS/How+to+do+Snapshotting+in+the+Mongo+Database>`_.
+          - `tailable` (optional): the result of this find call will be a
+            tailable cursor - tailable cursors aren't closed when the last data
+            is retrieved but are kept open and the cursors location marks the
+            final document's position. if more data is received iteration of
+            the cursor will continue from the last document received. For
+            details, see the `wiki
+            <http://www.mongodb.org/display/DOCS/Tailable+Cursors>`_.
         """
         if spec is None:
             spec = SON()
         if slave_okay is None:
             slave_okay = self.__database.connection().slave_okay
+        else:
+            warnings.warn("The slave_okay option to find and find_one is "
+                          "deprecated. Please set slave_okay on the Connection "
+                          "itself.",
+                          DeprecationWarning)
+
         if not isinstance(spec, types.DictType):
             raise TypeError("spec must be an instance of dict")
         if not isinstance(fields, (types.ListType, types.NoneType)):
@@ -369,14 +376,17 @@ class Collection(object):
             raise TypeError("timeout must be an instance of bool")
         if not isinstance(snapshot, types.BooleanType):
             raise TypeError("snapshot must be an instance of bool")
+        if not isinstance(tailable, types.BooleanType):
+            raise TypeError("tailable must be an instance of bool")
 
         if fields is not None:
             if not fields:
                 fields = ["_id"]
             fields = self._fields_list_to_dict(fields)
 
-        return Cursor(self, spec, fields, skip, limit, slave_okay, timeout, snapshot,
-                      _sock=_sock, _must_use_master=_must_use_master)
+        return Cursor(self, spec, fields, skip, limit, slave_okay, timeout,
+                      tailable, snapshot, _sock=_sock,
+                      _must_use_master=_must_use_master)
 
     def count(self):
         """Get the number of documents in this collection.
