@@ -25,9 +25,11 @@ import errno
 import datetime
 
 from errors import ConnectionFailure, ConfigurationError, AutoReconnect
+from errors import OperationFailure
 from database import Database
 from cursor_manager import CursorManager
 from thread_util import TimeoutableLock
+import bson
 
 _logger = logging.getLogger("pymongo.connection")
 _logger.addHandler(logging.StreamHandler())
@@ -36,6 +38,18 @@ _logger.setLevel(logging.INFO)
 _CONNECT_TIMEOUT = 20.0
 
 
+_LAST_ERROR_MESSAGE = (2004, # OP_QUERY
+                       "\x00\x00\x00\x00" # query options
+                       "admin.$cmd\x00" # collection name
+                       "\x00\x00\x00\x00" # skip
+                       "\xff\xff\xff\xff" + # limit (-1)
+                       bson.BSON.from_dict({"getlasterror": 1}))
+"""Data to send to do a lastError.
+
+This is a bit of a hack but we use it to do implement "safe" mode at a low
+level.
+"""
+
 class Connection(object): # TODO support auth for pooling
     """A connection to Mongo.
     """
@@ -50,7 +64,7 @@ class Connection(object): # TODO support auth for pooling
 
     def __init__(self, host=None, port=None, pool_size=None,
                  auto_start_request=None, timeout=None, slave_okay=False,
-                 _connect=True):
+                 network_timeout=None, _connect=True):
         """Open a new connection to a Mongo instance at host:port.
 
         The resultant connection object has connection-pooling built in. It
@@ -82,6 +96,8 @@ class Connection(object): # TODO support auth for pooling
           - `timeout` (optional): max time to wait when attempting to acquire a
             connection from the connection pool before raising an exception -
             can be set to -1 to wait indefinitely
+          - `network_timeout` (optional): timeout (in seconds) to use for socket
+            operations - default is no timeout
         """
         if host is None:
             host = self.HOST
@@ -128,6 +144,8 @@ class Connection(object): # TODO support auth for pooling
         self.__sockets = [None for _ in range(self.__pool_size)]
         self.__currently_resetting = False
 
+        self.__network_timeout = network_timeout
+
         # cache of existing indexes used by ensure_index ops
         self.__index_cache = {}
 
@@ -296,7 +314,7 @@ class Connection(object): # TODO support auth for pooling
                     sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                     sock.settimeout(_CONNECT_TIMEOUT)
                     sock.connect((host, port))
-                    sock.settimeout(None)
+                    sock.settimeout(self.__network_timeout)
                     master = self.__master(sock)
                     if master is True:
                         self.__host = host
@@ -350,7 +368,7 @@ class Connection(object): # TODO support auth for pooling
             sock = self.__sockets[socket_number]
             sock.settimeout(_CONNECT_TIMEOUT)
             sock.connect((self.host(), self.port()))
-            sock.settimeout(None)
+            sock.settimeout(self.__network_timeout)
             _logger.debug("connected")
             return
         except socket.error:
@@ -442,7 +460,11 @@ class Connection(object): # TODO support auth for pooling
             raise AutoReconnect(str(e))
         return sock
 
-    def __send_message_on_socket(self, operation, data, sock):
+    def __pack_message(self, operation, data):
+        """Takes message data and adds a message header based on the operation.
+
+        Returns the resultant (message, request_id) pair.
+        """
         # header
         request_id = self.__increment_id()
         to_send = struct.pack("<i", 16 + len(data))
@@ -450,12 +472,19 @@ class Connection(object): # TODO support auth for pooling
         to_send += "\x00\x00\x00\x00" # responseTo
         to_send += struct.pack("<i", operation)
 
-        to_send += data
+        return (to_send + data, request_id)
 
+    # TODO use static methods for a bunch of these
+    def __send_data_on_socket(self, data, sock):
+        """Lowest level send operation.
+
+        Takes data to send as a byte string and a socket instance and
+        sends all of the data, raising ConnectionFailure on error.
+        """
         total_sent = 0
-        while total_sent < len(to_send):
+        while total_sent < len(data):
             try:
-                sent = sock.send(to_send[total_sent:])
+                sent = sock.send(data[total_sent:])
             except socket.error, e:
                 if e[0] == errno.EAGAIN:
                     continue
@@ -464,55 +493,157 @@ class Connection(object): # TODO support auth for pooling
                 raise ConnectionFailure("connection closed, resetting")
             total_sent += sent
 
+    def __send_messages_on_socket(self, messages, sock):
+        """Pack and send a series of messages on the given socket.
+
+        `messages` should be a list of (operation, data) pairs. Returns the
+        request_id of the last message in the list.
+        """
+        packed = [self.__pack_message(op, data) for (op, data) in messages]
+        self.__send_data_on_socket("".join([d for (d, _) in packed]), sock)
+        return packed[-1][1]
+
+    def __send_message_on_socket(self, operation, data, sock):
+        """Pack and send a single message on the given socket.
+
+        Returns the resultant request_id.
+        """
+        (data, request_id) = self.__pack_message(operation, data)
+        self.__send_data_on_socket(data, sock)
         return request_id
 
-    def _send_message(self, operation, data):
+    # TODO does this really belong here?
+    def _unpack_response(response, cursor_id=None):
+        """Unpack a response from the database.
+
+        Check the response for errors and unpack, returning a dictionary
+        containing the response data.
+
+        :Parameters:
+          - `response`: byte string as returned from the database
+          - `cursor_id` (optional): cursor_id we sent to get this response -
+            used for raising an informative exception when we get cursor id not
+            valid at server response
+        """
+        response_flag = struct.unpack("<i", response[:4])[0]
+        if response_flag == 1:
+            # Shouldn't get this response if we aren't doing a getMore
+            assert cursor_id is not None
+
+            raise OperationFailure("cursor id '%s' not valid at server" %
+                                   cursor_id)
+        elif response_flag == 2:
+            error_object = bson.BSON(response[20:]).to_dict()
+            if error_object["$err"] == "not master":
+                db.connection()._reset()
+                raise AutoReconnect("master has changed")
+            raise OperationFailure("database error: %s" %
+                                   error_object["$err"])
+        else:
+            assert response_flag == 0
+
+        result = {}
+        result["cursor_id"] = struct.unpack("<q", response[4:12])[0]
+        result["starting_from"] = struct.unpack("<i", response[12:16])[0]
+        result["number_returned"] = struct.unpack("<i", response[16:20])[0]
+        result["data"] = bson._to_dicts(response[20:])
+        assert len(result["data"]) == result["number_returned"]
+        return result
+
+    _unpack_response = staticmethod(_unpack_response)
+
+    def __check_response_to_last_error(self, response):
+        """Check a response to a lastError message for errors.
+
+        `response` is a byte string representing a response to the message.
+        If it represents an error response we raise OperationFailure.
+        """
+        response = Connection._unpack_response(response)
+
+        assert response["number_returned"] == 1
+        error = response["data"][0]
+
+        # TODO unify logic with database.error method
+        if error.get("err", 0) is None:
+            return
+        if error["err"] == "not master":
+            self._reset()
+        raise OperationFailure(error["err"])
+
+    def _send_message(self, operation, data, safe=False):
         """Say something to Mongo.
 
-        Raises ConnectionFailure if the message cannot be sent. Returns the
-        request id of the sent message.
+        Raises ConnectionFailure if the message cannot be sent. Raises
+        OperationFailure if safe is True and the ensuing getLastError call
+        returns an error. Returns the request id of the sent message.
 
         :Parameters:
           - `operation`: opcode of the message
           - `data`: data to send
+          - `safe`: perform a getLastError after sending the message
         """
         sock_number = self.__get_socket()
         sock = self.__sockets[sock_number]
         try:
-            self.__send_message_on_socket(operation, data, sock)
+            try:
+                if not safe:
+                    return self.__send_message_on_socket(operation, data, sock)
+
+                # Safe mode. We pack the message together with a lastError
+                # message and send both. We then get the response (to the
+                # lastError) and raise OperationFailure if it is an error
+                # response.
+                messages = [(operation, data), _LAST_ERROR_MESSAGE]
+                request_id = self.__send_messages_on_socket(messages, sock)
+                response = self.__receive_message_on_socket(1, request_id, sock)
+                self.__check_response_to_last_error(response)
+            except ConnectionFailure, e:
+                self._reset()
+                raise AutoReconnect(str(e))
+        finally:
             self.__locks[sock_number].release()
-        except ConnectionFailure, e:
-            self.__locks[sock_number].release()
-            self._reset()
-            raise AutoReconnect(str(e))
+
+    def __receive_data_on_socket(self, length, sock):
+        """Lowest level receive operation.
+
+        Takes length to receive and repeatedly calls recv until able to
+        return a buffer of that length, raising ConnectionFailure on error.
+        """
+        message = ""
+        while len(message) < length:
+            try:
+                chunk = sock.recv(length - len(message))
+            except socket.error, e:
+                raise ConnectionFailure(e)
+            if chunk == "":
+                raise ConnectionFailure("connection closed")
+            message += chunk
+        return message
 
     def __receive_message_on_socket(self, operation, request_id, sock):
+        """Receive a message in response to `request_id` on `sock`.
 
-        def receive(length):
-            message = ""
-            while len(message) < length:
-                try:
-                    chunk = sock.recv(length - len(message))
-                except socket.error:
-                    raise ConnectionFailure("connection closed")
-                if chunk == "":
-                    raise ConnectionFailure("connection closed")
-                message += chunk
-            return message
-
-        header = receive(16)
+        Returns the response data with the header removed.
+        """
+        header = self.__receive_data_on_socket(16, sock)
         length = struct.unpack("<i", header[:4])[0]
         assert request_id == struct.unpack("<i", header[8:12])[0]
         assert operation == struct.unpack("<i", header[12:])[0]
 
-        return receive(length - 16)
+        return self.__receive_data_on_socket(length - 16, sock)
+
+    def __send_and_receive(self, operation, data, sock):
+        """Send a message on the given socket and return the response data.
+        """
+        request_id = self.__send_message_on_socket(operation, data, sock)
+        return self.__receive_message_on_socket(1, request_id, sock)
 
     __hack_socket_lock = threading.Lock()
-
     # we just ignore _must_use_master here: it's only relavant for
     # MasterSlaveConnection instances.
-    def _receive_message(self, operation, data, _sock=None, _must_use_master=False):
-        """Receive a message from Mongo.
+    def _send_message_with_response(self, operation, data,
+                                    _sock=None, _must_use_master=False):
+        """Send a message to Mongo and return the response.
 
         Sends the given message and returns the response.
 
@@ -524,24 +655,20 @@ class Connection(object): # TODO support auth for pooling
         if _sock:
             self.__hack_socket_lock.acquire()
             try:
-                request_id = self.__send_message_on_socket(operation,
-                                                           data, _sock)
-                result = self.__receive_message_on_socket(1, request_id, _sock)
-                return result
+                return self.__send_and_receive(operation, data, _sock)
             finally:
                 self.__hack_socket_lock.release()
 
         sock_number = self.__get_socket()
         sock = self.__sockets[sock_number]
         try:
-            request_id = self.__send_message_on_socket(operation, data, sock)
-            result = self.__receive_message_on_socket(1, request_id, sock)
+            try:
+                return self.__send_and_receive(operation, data, sock)
+            except ConnectionFailure, e:
+                self._reset()
+                raise AutoReconnect(str(e))
+        finally:
             self.__locks[sock_number].release()
-            return result
-        except ConnectionFailure, e:
-            self.__locks[sock_number].release()
-            self._reset()
-            raise AutoReconnect(str(e))
 
     def start_request(self):
         """Start a "request".
