@@ -37,7 +37,8 @@ class Cursor(object):
     """
 
     def __init__(self, collection, spec, fields, skip, limit, slave_okay,
-                 timeout, snapshot=False, _sock=None, _must_use_master=False):
+                 timeout, tailable, snapshot=False,
+                 _sock=None, _must_use_master=False):
         """Create a new cursor.
 
         Should not be called directly by application developers.
@@ -49,6 +50,7 @@ class Cursor(object):
         self.__limit = limit
         self.__slave_okay = slave_okay
         self.__timeout = timeout
+        self.__tailable = tailable
         self.__snapshot = snapshot
         self.__ordering = None
         self.__explain = False
@@ -62,6 +64,12 @@ class Cursor(object):
         self.__retrieved = 0
         self.__killed = False
 
+    def collection(self):
+        """Get the collection for this cursor.
+        """
+        return self.__collection
+    collection = property(collection)
+
     def __del__(self):
         if self.__id and not self.__killed:
             self.__die()
@@ -93,7 +101,7 @@ class Cursor(object):
         """
         copy = Cursor(self.__collection, self.__spec, self.__fields,
                       self.__skip, self.__limit, self.__slave_okay,
-                      self.__timeout, self.__snapshot)
+                      self.__timeout, self.__tailable, self.__snapshot)
         copy.__ordering = self.__ordering
         copy.__explain = self.__explain
         copy.__hint = self.__hint
@@ -135,6 +143,8 @@ class Cursor(object):
         """Get the 4 byte query options string to use for this query.
         """
         options = 0
+        if self.__tailable:
+            options |= _QUERY_OPTIONS["tailable_cursor"]
         if self.__slave_okay:
             options |= _QUERY_OPTIONS["slave_okay"]
         if not self.__timeout:
@@ -354,6 +364,38 @@ class Cursor(object):
         self.__spec["$where"] = code
         return self
 
+    def __send_message(self, operation, message):
+        """Send a query or getmore message and handles the response.
+        """
+        db = self.__collection.database()
+        kwargs = {"_sock": self.__socket,
+                  "_must_use_master": self.__must_use_master}
+        if self.__connection_id is not None:
+            kwargs["_connection_to_use"] = self.__connection_id
+
+        response = db.connection()._send_message_with_response(operation, message,
+                                                               **kwargs)
+
+        if isinstance(response, types.TupleType):
+            (connection_id, response) = response
+        else:
+            connection_id = None
+
+        self.__connection_id = connection_id
+
+        response = pymongo.Connection._unpack_response(response, self.__id)
+        self.__id = response["cursor_id"]
+
+        # starting from doesn't get set on getmore's for tailable cursors
+        if not self.__tailable:
+            assert response["starting_from"] == self.__retrieved
+
+        self.__retrieved += response["number_returned"]
+        self.__data = response["data"]
+
+        if self.__limit and self.__id and self.__limit <= self.__retrieved:
+            self.__die()
+
     def _refresh(self):
         """Refreshes the cursor with more data from Mongo.
 
@@ -364,48 +406,6 @@ class Cursor(object):
         if len(self.__data) or self.__killed:
             return len(self.__data)
 
-        def send_message(operation, message):
-            db = self.__collection.database()
-            kwargs = {"_sock": self.__socket,
-                      "_must_use_master": self.__must_use_master}
-            if self.__connection_id is not None:
-                kwargs["_connection_to_use"] = self.__connection_id
-
-            response = db.connection()._receive_message(operation, message,
-                                                        **kwargs)
-
-            if isinstance(response, types.TupleType):
-                (connection_id, response) = response
-            else:
-                connection_id = None
-
-            response_flag = struct.unpack("<i", response[:4])[0]
-            if response_flag == 1:
-                raise OperationFailure("cursor id '%s' not valid at server" %
-                                       self.__id)
-            elif response_flag == 2:
-                error_object = bson.BSON(response[20:]).to_dict()
-                if error_object["$err"] == "not master":
-                    db.connection()._reset()
-                    raise AutoReconnect("master has changed")
-                raise OperationFailure("database error: %s" %
-                                       error_object["$err"])
-            else:
-                assert response_flag == 0
-
-            self.__id = struct.unpack("<q", response[4:12])[0]
-            self.__connection_id = connection_id
-            assert struct.unpack("<i", response[12:16])[0] == self.__retrieved
-
-            number_returned = struct.unpack("<i", response[16:20])[0]
-            self.__retrieved += number_returned
-
-            if self.__limit and self.__id and self.__limit <= self.__retrieved:
-                self.__die()
-
-            self.__data = bson._to_dicts(response[20:])
-            assert len(self.__data) == number_returned
-
         message = self.__query_options()
         message += bson._make_c_string(self.__collection.full_name())
         if self.__id is None:
@@ -416,7 +416,7 @@ class Cursor(object):
             if self.__fields:
                 message += bson.BSON.from_dict(self.__fields)
 
-            send_message(2004, message)
+            self.__send_message(2004, message)
             if not self.__id:
                 self.__killed = True
         elif self.__id:
@@ -432,7 +432,7 @@ class Cursor(object):
             message += struct.pack("<i", limit)
             message += struct.pack("<q", self.__id)
 
-            send_message(2005, message)
+            self.__send_message(2005, message)
 
         return len(self.__data)
 
