@@ -57,7 +57,6 @@ class Database(object):
         self.__outgoing_manipulators = []
         self.__outgoing_copying_manipulators = []
         self.add_son_manipulator(ObjectIdInjector())
-        self.add_son_manipulator(ObjectIdShuffler())
 
     def __check_name(self, name):
         for invalid_char in [" ", ".", "$", "/", "\\"]:
@@ -83,7 +82,7 @@ class Database(object):
             if method_overwritten(manipulator, "transform_incoming"):
                 self.__incoming_copying_manipulators.insert(0, manipulator)
             if method_overwritten(manipulator, "transform_outgoing"):
-                self.__outgoing_manipulators.insert(0, manipulator)
+                self.__outgoing_copying_manipulators.insert(0, manipulator)
         else:
             if method_overwritten(manipulator, "transform_incoming"):
                 self.__incoming_manipulators.insert(0, manipulator)
@@ -182,7 +181,9 @@ class Database(object):
     def _command(self, command, allowable_errors=[], check=True, sock=None):
         """Issue a DB command.
         """
-        result = self["$cmd"].find_one(command, _sock=sock)
+        result = self["$cmd"].find_one(command, _sock=sock,
+                                       _must_use_master=True)
+
         if check and result["ok"] != 1:
             if result["errmsg"] in allowable_errors:
                 return result
@@ -193,7 +194,7 @@ class Database(object):
     def collection_names(self):
         """Get a list of all the collection names in this database.
         """
-        results = self["system.namespaces"].find()
+        results = self["system.namespaces"].find(_must_use_master=True)
         names = [r["name"] for r in results]
         names = [n[len(self.__name) + 1:] for n in names
                  if n.startswith(self.__name + ".")]
