@@ -15,6 +15,7 @@
 """Test the collection module."""
 import unittest
 import re
+import itertools
 import time
 import sys
 sys.path[0:0] = [""]
@@ -405,6 +406,23 @@ class TestCollection(unittest.TestCase):
         self.assertEqual(doc1["_id"], ids[0])
         self.assertEqual(doc2["_id"], ids[1])
 
+    def test_insert_iterables(self):
+        db = self.db
+
+        self.assertRaises(TypeError, db.test.insert, 4)
+        self.assertRaises(TypeError, db.test.insert, None)
+        self.assertRaises(TypeError, db.test.insert, True)
+
+        db.drop_collection("test")
+        self.assertEqual(db.test.find().count(), 0)
+        ids = db.test.insert(({"hello": u"world"}, {"hello": u"world"}))
+        self.assertEqual(db.test.find().count(), 2)
+
+        db.drop_collection("test")
+        self.assertEqual(db.test.find().count(), 0)
+        ids = db.test.insert(itertools.imap(lambda x: {"hello": "world"}, itertools.repeat(None, 10)))
+        self.assertEqual(db.test.find().count(), 10)
+
     def test_save(self):
         self.db.drop_collection("test")
         id = self.db.test.save({"hello": "world"})
@@ -483,17 +501,22 @@ class TestCollection(unittest.TestCase):
 
     def test_safe_update(self):
         db = self.db
+        v11 = version.at_least(db.connection(), (1, 1))
+
         db.drop_collection("test")
-        db.test.create_index("x")
+        db.test.create_index("x", unique=True)
 
-        a = {"x": 5}
-        db.test.insert(a)
+        db.test.insert({"x": 5})
+        id = db.test.insert({"x": 4})
 
-        db.test.update({}, {"$inc": {"x": 1}})
-        self.assert_(db.error()["err"].startswith("E12011"))
+        db.test.update({"_id": id}, {"$inc": {"x": 1}})
+        if v11:
+            self.assert_(db.error()["err"].startswith("E11001"))
+        else:
+            self.assert_(db.error()["err"].startswith("E12011"))
 
         self.assertRaises(OperationFailure, db.test.update,
-                          {}, {"$inc": {"x": 1}}, safe=True)
+                          {"_id": id}, {"$inc": {"x": 1}}, safe=True)
 
     def test_safe_save(self):
         db = self.db
@@ -506,6 +529,20 @@ class TestCollection(unittest.TestCase):
 
         self.assertRaises(OperationFailure, db.test.save, {"hello": "world"}, safe=True)
 
+    def test_safe_remove(self):
+        db = self.db
+        db.drop_collection("test")
+        db.create_collection("test", {"capped": True, "size": 1000})
+
+        db.test.insert({"x": 1})
+        self.assertEqual(1, db.test.count())
+
+        db.test.remove({"x": 1})
+        self.assertEqual(1, db.test.count())
+
+        # TODO test this raises an exception (waiting on db support for reporting this)
+        db.test.remove({"x": 1}, safe=True)
+
     def test_count(self):
         db = self.db
         db.drop_collection("test")
