@@ -27,6 +27,7 @@ from pymongo.database import Database
 from pymongo.code import Code
 from pymongo import ASCENDING, DESCENDING
 from test_connection import get_connection
+import version
 
 
 class TestCursor(unittest.TestCase):
@@ -82,19 +83,20 @@ class TestCursor(unittest.TestCase):
 
         self.assertRaises(TypeError, db.test.find().hint, index)
 
-    # TODO right now this doesn't actually test anything useful, just that the
-    # call doesn't blow up in the normal case.
+
+    # This is deprecated - test that a warning is actually raised
     def test_slave_okay(self):
         db = self.db
         db.drop_collection("test")
-
-        a = db.test.find(slave_okay=True)
-        for _ in a:
-            break
-
         db.test.save({"x": 1})
-        self.assertEqual(1, db.test.find(slave_okay=True).next()["x"])
-        self.assertEqual(1, db.test.find(slave_okay=False).next()["x"])
+
+        warnings.simplefilter("error")
+
+        self.assertEqual(1, db.test.find().next()["x"])
+        self.assertRaises(DeprecationWarning, db.test.find, slave_okay=True)
+        self.assertRaises(DeprecationWarning, db.test.find, slave_okay=False)
+
+        warnings.simplefilter("default")
 
     def test_limit(self):
         db = self.db
@@ -442,10 +444,13 @@ class TestCursor(unittest.TestCase):
         self.db.test.remove({})
         self.db.test.save({"x": 1})
 
-        for _ in self.db.test.find({}, ["a"]):
-            self.fail()
+        if not version.at_least(self.db.connection(), (1, 1)):
+            for _ in self.db.test.find({}, ["a"]):
+                self.fail()
 
-        self.assertEqual(0, self.db.test.find({}, ["a"]).count())
+            self.assertEqual(0, self.db.test.find({}, ["a"]).count())
+        else:
+            self.assertEqual(1, self.db.test.find({}, ["a"]).count())
 
     def test_bad_getitem(self):
         self.assertRaises(TypeError, lambda x: self.db.test.find()[x], "hello")
@@ -542,6 +547,43 @@ class TestCursor(unittest.TestCase):
         check_len(self.db.test.find().limit(10).skip(95), 5)
         check_len(self.db.test.find()[95:105], 5)
 
+    def test_properties(self):
+        self.assertEqual(self.db.test, self.db.test.find().collection)
+
+        def set_coll():
+            self.db.test.find().collection = "hello"
+
+        self.assertRaises(AttributeError, set_coll)
+
+    def test_tailable(self):
+        db = self.db
+        db.drop_collection("test")
+
+        cursor = db.test.find(tailable=True)
+
+        db.test.insert({"x": 1})
+        count = 0
+        for doc in cursor:
+            count += 1
+            self.assertEqual(1, doc["x"])
+        self.assertEqual(1, count)
+
+        db.test.insert({"x": 2})
+        count = 0
+        for doc in cursor:
+            count += 1
+            self.assertEqual(2, doc["x"])
+        self.assertEqual(1, count)
+
+        db.test.insert({"x": 3})
+        count = 0
+        for doc in cursor:
+            count += 1
+            self.assertEqual(3, doc["x"])
+        self.assertEqual(1, count)
+
+        self.assertEqual(3, db.test.count())
+
 
 if __name__ == "__main__":
     unittest.main()
